#include "arguments.h"

#include <string.h>
#include <stdio.h>

static const Argument *findShortArgument(const char *text, const Argument *start, const Argument *end, const char **text_end)
 {
  const Argument *p;
  
  for(p = start; p != end; ++p)
   if(*text == p->short_name)
    {
     *text_end = text+1;
     if(!**text_end) *text_end = 0;
     
     return p;
    }
  
  return 0;
 }

static const Argument *findLongArgument(const char *text, const Argument *start, const Argument *end, const char **text_end)
 {
  const Argument *p;
  const char *e = text;
  size_t len;
  
  while(*e && *e != '=')
   ++e;
  
  len = e-text;
  
  for(p = start; p != end; ++p)
   if(p->long_name && len == strlen(p->long_name) && !memcmp(text, p->long_name, len))
    {
     if(*e == '=')
      *text_end = e;
     else
      *text_end = 0;
     
     return p;
    }
  
  return 0;
 }

static const Argument *findFileArgument(const Argument *start, const Argument *end)
 {
  const Argument *p;
  
  for(p = start; p != end; ++p)
   if(!p->long_name && !p->short_name)
    return p;
  
  return 0;
 }

static const char *getArgumentName(const Argument *arg)
 {
  if(arg->long_name)
   return arg->long_name;
  
  if(!arg->short_name)
   return "<file>";
  
  static char buf[2] = {0, 0};
  buf[0] = arg->short_name;
  return buf;
 }

int parseArguments(int argc, const char * const *argv, const Argument *start, const Argument *end, void *data)
 {
  if(argc < 1) return 0;
  
  //const char *program = *argv;
  
  for(--argc, ++argv; argc; --argc, ++argv)
   {
    const char *arg = *argv;
    char force_short = 0;
    
    while(arg && *arg)
     {
      const Argument *argument;
      
      if(force_short)
       argument = findShortArgument(arg, start, end, &arg);
      else if(*arg == '-')
       {
        if(arg[1] == '-')
         argument = findLongArgument(arg+2, start, end, &arg);
        else if(arg[1])
         {
          force_short = 1;
          argument = findShortArgument(arg+1, start, end, &arg);
         }
        else
         {
          fprintf(stderr, "Expected an option name.\n");
          return 0;
         }
       }
      else argument = findFileArgument(start, end);
      
      if(!argument)
       {
        fprintf(stderr, "Don't know how to handle '%s'.\n", arg ? arg : *argv);
        return 0;
       }
      
      if(arg && *arg == '=')
       {
        if(argument->type == NoArgument)
         {
          fprintf(stderr, "Didn't expect argument for option %s.\n", getArgumentName(argument));
          return 0;
         }
        
        if(!(*argument->func)(arg+1, data))
         return 0;
        
        arg = 0;
       }
      else if(argument->type == RequireArgument)
       {
        if(!arg)
         {
          --argc; ++argv;
          
          if(!argc)
           {
            fprintf(stderr, "Expected an argument for option %s.\n", getArgumentName(argument));
            return 0;
           }
          
          arg = *argv;
         }
        
        if(!(*argument->func)(arg, data))
         return 0;
        
        arg = 0;
       }
      else if(!(*argument->func)("", data))
       return 0;
     }
   }
  
  return 1;
 }
