#ifndef GUI_H
#define GUI_H

#include <allegro.h>
#include "region.h"

extern const unsigned int attr_dirty;
// extern const unsigned int attr_redraw;
extern const unsigned int attr_select;
extern const unsigned int attr_move;
extern const unsigned int attr_resize;
extern const unsigned int attr_masked;

typedef struct Window
 {
  struct Window *parent, **start, **end, **alloc;
  BITMAP *buffer;
  int x, y, real_x, real_y; // x,y is relative to the parent, real_x,real_y is relative to the screen.
  unsigned int attr;
  
  int (*update)(struct Window *); // Returns non-zero if everything is okay, 0 if it wants to die.
  int (*mouse)(struct Window *, int x, int y, int xoff, int yoff, int b, int b_off); // Returns non-zero if the movement was used.
  void (*redraw)(struct Window *);
  int (*key)(struct Window *, int value, int scancode); // Returns non-zero if the key was used.
  void (*destroy)(struct Window *);
  
  void *data; // A window may store extra data here.
  
  DirtyRectangles dr;
 } Window;

extern Window *root; // This is the root window where everything is drawn on.

extern int GUIInit(int width, int height, int depth, int fullscreen);
extern void GUIUninit(void);
extern void GUIUpdate(void);

extern Window *GUICreateWindow(int (*init)(Window *, void *), void *init_data, int x, int y, int w, int h);
extern void GUIDestroyWindow(Window *window);

extern int GUIAddRoot(Window *w);
extern void GUIRemoveRoot(Window *w);
extern int GUIAddChild(Window *w, Window *child);
extern void GUIRemoveChild(Window *w, Window *child);

extern void GUIMarkRedraw(Window *w);
extern void GUIMarkDirty(Window * w, const Region *r);
extern void GUIMarkDirtyGroup(Window * w, const DirtyRectangles *g);
extern void GUIMarkDirtyAll(Window *w);

extern void GUIMoveWindow(Window *w, int xoff, int yoff);
extern void GUIPositionWindow(Window *w, int x, int y);
extern int GUIResizeWindow(Window *w, int width, int height);

#include "guibits.h"

#endif
